/*
    This file is part of Extend Panel Menu

    Extend Panel Menu is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Extend Panel Menu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Extend Panel Menu.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2017 Julio Galvan
*/

const St = imports.gi.St;
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Shell = imports.gi.Shell;
const Main = imports.ui.main;
const Mainloop = imports.mainloop;
const Clutter = imports.gi.Clutter;
const Tweener = imports.ui.tweener;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;

var CustomButton = new Lang.Class({
    Name: "Button",
    Extends: PanelMenu.Button,

    _init: function (name) {
        this.parent(0.4, name);
        this.name = name;
        this._showTipTimeoutId = 0;
        this._center = false;
        this.box = new St.BoxLayout({
            vertical: false,
            style_class: "panel-status-menu-box"
        });;
        this.actor.add_child(this.box);
    },
    _onEvent: function(actor, event){
        if (this.menu &&
            (event.type() == Clutter.EventType.BUTTON_RELEASE)) {
           let button = event.get_button();
           if(1== button)
           {
                this.menu.toggle();
                if (this._showTipTimeoutId > 0)
                    Mainloop.source_remove(this._showTipTimeoutId);
                this._showTipTimeoutId = 0;
                this._hideTip();
           }
        } else if (event.type() == Clutter.EventType.ENTER) {
            this.actor.add_style_class_name('panel-button-hover');
            if(!this.menu.isOpen) {
                if (this._showTipTimeoutId == 0) {
                    this._showTipTimeoutId = Mainloop.timeout_add(800, 
                        Lang.bind(this, function(){
                            this._showTip();
                            this._showTipTimeoutId = 0;
                            return GLib.SOURCE_REMOVE;
                    }));
                }
            }
        } else if (event.type() == Clutter.EventType.LEAVE) {
            if(this.actor.has_style_class_name('panel-button-hover'))
                this.actor.remove_style_class_name('panel-button-hover');
            if (this._showTipTimeoutId > 0)
                Mainloop.source_remove(this._showTipTimeoutId);
            this._showTipTimeoutId = 0;
            this._hideTip();
        }
        return Clutter.EVENT_STOP;
    },
    _hideTip: function() {
        if(this._text && this._text.get_parent())
        {
               Tweener.addTween(this._text,
                         { opacity: 0,
                           time: 0.5,
                           transition: 'easeOutQuad'
                           });
               Main.uiGroup.remove_actor(this._text);
               this._text.destroy();
               this._text = null;
        }
    },
    _showTip: function() {
       if (!this._text) {
           this._text = new St.Label({style_class:"dash-label",text: this.name });
           Main.uiGroup.add_actor(this._text);
       }
       let [x,y]=this.actor.get_transformed_position();
       let [a,b]=this._text.get_transformed_size();
       this._text.opacity = 0;
       let monitor = Main.layoutManager.primaryMonitor;
       if ( x + a > monitor.width) {
           x = monitor.width - a;
       }
       this._text.set_position(x,y-35);

       Tweener.addTween(this._text,
                     { opacity: 255,
                       time: 0.5,
                       transition: 'easeOutQuad'
                       });

    },

    _openApp: function (a, b, app) {
        Shell.AppSystem.get_default().lookup_app(app).activate();
    },
    set_spacing: function (spacing) {
        this._default_spacing = spacing;
        this.update_spacing(spacing);
    },
    update_spacing: function (spacing) {
        let style = '-natural-hpadding: %dpx'.format(spacing);
        if (spacing < 6) {
            style += '; -minimum-hpadding: %dpx'.format(spacing);
        }
        this.actor.set_style(style);
    },
    calculate_spacing: function () {
        let style = this.actor.get_style();
        if (style) {
            let start = style.indexOf("-natural-hpadding: ");
            let end = style.indexOf("px;");
            let val = parseInt(style.substring(start + 19, end));
            return val;
        }
        return NaN
    },
    destroy: function () {
        this.parent();
    }
});
