const Atk = imports.gi.Atk;
const Cairo = imports.cairo;
const Clutter = imports.gi.Clutter;
const Lang = imports.lang;
const St = imports.gi.St;
const Signals = imports.signals;

const ExtensionUtils = imports.misc.extensionUtils;
const Slider = imports.ui.slider;

//Class of the Kylin slider
const VSlider = new Lang.Class({
    Name: "VSlider",
    Extends: Slider.Slider,

    _init: function(value){
        this.parent(value);
        
    },

    _sliderRepaint: function(area) {
        let cr = area.get_context();
        let themeNode = area.get_theme_node();
        let [width, height] = area.get_surface_size();

        let handleRadius = themeNode.get_length('-slider-handle-radius');

        let handleBorderWidth = themeNode.get_length('-slider-handle-border-width');
        let [hasHandleColor, handleBorderColor] =
            themeNode.lookup_color('-slider-handle-border-color', false);

        let sliderHeight = themeNode.get_length('-slider-height');

        let sliderBorderWidth = themeNode.get_length('-slider-border-width');
        let sliderBorderRadius = Math.min(width, sliderHeight) / 2;

        let sliderBorderColor = themeNode.get_color('-slider-border-color');
        let sliderColor = themeNode.get_color('-slider-background-color');

        let sliderActiveBorderColor = themeNode.get_color('-slider-active-border-color');
        let sliderActiveColor = themeNode.get_color('-slider-active-background-color');

        const TAU = Math.PI * 2;

        let handleX = handleRadius + (height - 2 * handleRadius) * ( 1 - this._value);

        cr.arc(2+ sliderHeight / 2 + handleRadius, height - sliderBorderRadius - sliderBorderWidth, sliderBorderRadius, TAU, TAU * 1/2);
        cr.lineTo(2+0 + handleRadius, handleX);
        cr.lineTo(2+ sliderHeight + handleRadius, handleX);
        cr.lineTo(2+ sliderHeight + handleRadius, height - sliderBorderRadius - sliderBorderWidth);
        Clutter.cairo_set_source_color(cr, sliderActiveColor);
        cr.fillPreserve();
        Clutter.cairo_set_source_color(cr, sliderActiveBorderColor);
        cr.setLineWidth(sliderBorderWidth);
        cr.stroke();

        cr.arc(2+ sliderHeight / 2 + handleRadius, sliderBorderRadius + sliderBorderWidth, sliderBorderRadius, TAU * 1/2, TAU);
        cr.lineTo(2+ sliderHeight + handleRadius, handleX);
        cr.lineTo(2+0 + handleRadius, handleX);
        cr.lineTo(2+0 + handleRadius, sliderBorderRadius + sliderBorderWidth);
        Clutter.cairo_set_source_color(cr, sliderColor);
        cr.fillPreserve();
        Clutter.cairo_set_source_color(cr, sliderBorderColor);
        cr.setLineWidth(sliderBorderWidth);
        cr.stroke();

        let handleY = 2+ sliderHeight / 2 + handleRadius;

        let color = themeNode.get_foreground_color();
        Clutter.cairo_set_source_color(cr, color);
        cr.arc(handleY, handleX, handleRadius, 0, 2 * Math.PI);
        cr.fillPreserve();
        if (hasHandleColor && handleBorderWidth) {
            Clutter.cairo_set_source_color(cr, handleBorderColor);
            cr.setLineWidth(handleBorderWidth);
            cr.stroke();
        }
        cr.$dispose();
    },

    _moveHandle: function(absX, absY) {
        let relX, relY, sliderX, sliderY;
        [sliderX, sliderY] = this.actor.get_transformed_position();
        relY = absX - sliderX;
        relX = absY - sliderY;

        let width = this.actor.height;
        let handleRadius = this.actor.get_theme_node().get_length('-slider-handle-radius');

        let newvalue;
        if (relX < handleRadius)
            newvalue = 0;
        else if (relX > width - handleRadius)
            newvalue = 1;
        else
            newvalue = (relX - handleRadius) / (width - 2 * handleRadius);
        this._value =  1 - newvalue;
        this.actor.queue_repaint();
        this.emit('value-changed', this._value);
    },

    destroy: function() {
        this.parent();
    }

});
